/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : VRPSolution.h                                            */
/*                                                                       */
/*       Date : May   5, 1999                                            */
/* Updated at : June 26, 1999                                            */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/**
<changelog>
 <log>
  <date>7.26.1999</date>
  <desc>pickAnEdge() is added for Random and Greedy choice:cost_per_demand</desc>
 </log>
</changelog>
**/
/*************************************************************************/

#ifndef _VRPSOLUTION_H_
#define _VRPSOLUTION_H_

class VRPSolution;

#include "VRPcommons.h"
#include "VRProblem.h"

#include "EdgeList.h"
#include "ListOfEdgeLists.h"
#include "Cover.h"

class VRPSolution
{  
   public :

     VRPSolution(VRProblem* _myProblem);
     VRPSolution();

     ~VRPSolution();

     // copy constructor
     VRPSolution(const VRPSolution&);

     void setProblem(VRProblem* _myProblem) ;
     inline VRProblem* getProblem() { return myProblem; };

     inline getNumberOfVehicles() const { return myEdgeLists.getSize(); };
     inline getNumOfRotations() const { return getNumberOfVehicles(); };

     void addVehicleSchedule(ListOfEdgeLists& l);
     inline void addListOfEdgeList(ListOfEdgeLists& l) 
     { addVehicleSchedule(l); }

     inline ListOfEdgeListsPtr getListOfEdgeListsIndex(int indx) const {  
       ListOfEdgeListsPtr ret=NULL;
       if(indx>=0&&indx<getNumberOfVehicles()) {
          ListOfEdgeListsPtr* dummy=myEdgeLists.getNthElement(indx);
	  if(dummy!=NULL) ret=*dummy;
       }// end of if
       return ret;
	//(indx>=0&&indx<getNumberOfVehicles()?*myEdgeLists.getNthElement(indx):NULL); } ;
     }// end of getListOfEdgeListsIndex() 

     void addVehicleDuty(EdgeListPtr eL);

     inline isCustomerServed(int indx) const
     { return (myCover!=NULL?myCover->isCovered(indx):0);}

     solveRandom();

     void cleanUpVehicles();

     // assignment operator
     VRPSolution& operator=(const VRPSolution& rhs);

     // equal?
     operator==(const VRPSolution& rhs);

     // print 
     friend ostream& operator<<(ostream& os,const VRPSolution& rhs);

     void nicePrint(ostream& os) const ;

     EdgeListPtr getAllEdges() const { return NULL; }

     inline vrpcost getCost() const { return cost; }

     inline EdgeGraphPtr getEdgeGraph() const 
     { return (myProblem!=NULL?myProblem->getEdgeGraph():NULL); }

     void writeStat(ostream& os) ;

     // Serialization Interface
     void writeCoded(ostream& os) ;
     void readCoded(istream& os) ;

     void updateVehicles(ListOfEdgeListsPtr lofel);
     void patchUp(tSet<EdgePtr>* abandoned=NULL) ;
     void applyLocalSearch();

     doesContainEdge(int from, int to) const;

     enum SelectStrategy { RANDOM, GREEDY, DISTGREEDY, WTIME, WTPERDEMAND};
   private:
  gVector<ListOfEdgeListsPtr>  myEdgeLists;
                       Cover*  myCover;
		      vrpcost  cost;
		  vrpcapacity  tdemand;
		      vrptime  wtime;

     void dispose();
     void copyFrom(const VRPSolution&);
     canMerge(EdgeListPtr edgeList);
     void patchUpRandom(Cover, tSet<EdgePtr>* abondoned=NULL );

   protected:
                   VRProblem*  myProblem;

    EdgeListPtr preProcess(EdgeListPtr eList);
    Edge pickAnEdge(WeightedRandomPickerSet<EdgePtr>* leaving, SelectStrategy strategy=RANDOM); 
    Edge VRPSolution::pickAnEdgeWithRndSS(WeightedRandomPickerSet<EdgePtr>* leaving);



}; // end of class VRPSolution

#endif
